/*
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/**
 * \file    DM388_i2c.c
 *
 * \brief   I2C routines for DM388 EVM
 *
 * This file contains the routines for configuring and
 * accessing the I2C controller and slave devices.
 *
 * \author  0043
 *
 * \version 01a,13aug,2010 Created
 */

/*
 *====================
 * Includes
 *====================
 */
#include "dm388_types.h"
#include "dm388_i2c.h"
#include "dm388_EVM.h"

#include "stdio.h"

/*
 *====================
 * Defines
 *====================
*/
#define I2C_FUNC_CLOCK		(48000000)
#define I2C_BUS_CLOCK		(100000)
#define DM388_I2C_TIMEOUT	(1000000)

//#define CTRL_MODULE_BASE_ADDR	(0x48140000u)

/* I2C pins for I2C - 2 */
//#define PINCNTL74				(CTRL_MODULE_BASE_ADDR + 0x0924)
//#define PINCNTL75				(CTRL_MODULE_BASE_ADDR + 0x0928)

//#define PINCNTL263				(CTRL_MODULE_BASE_ADDR + 0x0C18)
//#define PINCNTL264				(CTRL_MODULE_BASE_ADDR + 0x0C1C)

/*
 *====================
 * Static declarations
 *====================
 */

/*
 *====================
 * Function implementations
 *====================
 */
/** **************************************************************************
 * \n \brief Routine to initialize the I2C - 0
 *
 * This routine initializes the I2C instance 0. We are not configuring the 
 * interrupts here as the code is intended to carry out all I/O operations
 * in polled mode.
 *
 * \param u8BusNum		[IN]	Bus Instance number
 *
 * \return
 * \n      return SUCCESS for success  - Description
 * \n      return FAILED for error   - Description
 */
INT32 DM388I2cInit
(
	UINT8	u8BusNum
)
{
	INT32	u32RetVal = SUCCESS;
	UINT16	u16SlaveAddr = 0x20;
	DM388_I2C_REGS	stI2cRegs = NULL;
	UINT16	u16DelayCnt = 1000;
	UINT16  u16SCLValues = 0;
	
	switch (u8BusNum)
	{
		case 0:
			stI2cRegs = (DM388_I2C_REGS) I2C0_BASE;
			break;
		case 1:
			stI2cRegs = (DM388_I2C_REGS) I2C1_BASE;
			break;
		case 2:
			stI2cRegs = (DM388_I2C_REGS) I2C2_BASE;
			break;
		case 3:
			stI2cRegs = (DM388_I2C_REGS) I2C3_BASE;
			break;
		default:
			stI2cRegs = (DM388_I2C_REGS) NULL;
			platform_write ("Invalid I2C bus number \r\n");
			return FAILED;
	}
	
	/* Ensure the module is enabled and the clock to it is also enabled */
	*(UINT32*) CM_ALWON_I2C_0_CLKCTRL = 0x02;
	
	DM388_wait(1000);
	
	/* Check the state is changed */
	while ((*(UINT32*) CM_ALWON_I2C_0_CLKCTRL) & 0x00030000);

	/* Ensure the module is enabled and the clock to it is also enabled */
	*(UINT32*) CM_ALWON_I2C_1_CLKCTRL = 0x02;
	
	DM388_wait(1000);
	
	/* Check the state is changed */
	while ((*(UINT32*) CM_ALWON_I2C_1_CLKCTRL) & 0x00030000);


	/* Reset all the registers */
	stI2cRegs->I2C_CON = 0;
	stI2cRegs->I2C_PSC = 0;
	stI2cRegs->I2C_SCLL = 0;
	stI2cRegs->I2C_SCLH = 0;
	stI2cRegs->I2C_BUF = 0;
	stI2cRegs->I2C_SYSC = 0;
	stI2cRegs->I2C_WE = 0;
	stI2cRegs->I2C_STAT = 0xFFFF;
		
	/* DM388 */
	DM388_usecDelay (500);

	/* soft-reset the I2c Controller */
	stI2cRegs->I2C_SYSC = SYSC_SOFTRESET_MASK; 
	
	/* For some reasons, the I2C requires to be enabled for reset to complete */
	stI2cRegs->I2C_CON = I2C_CON_EN;
	
	/* wait till reset is done */
	while (SYSS_RESETDONE_MASK != (stI2cRegs->I2C_SYSC & SYSS_RESETDONE_MASK))
	{
		u16DelayCnt--;
		DM388_usecDelay(1);
		
		if (u16DelayCnt <= 0)
			break;
	}
	
	if (u16DelayCnt == 0)
	{
		u32RetVal = FAILED;
		return (u32RetVal);
	}
	
	/* SYSC register is cleared by the reset, rewrite it */
	stI2cRegs->I2C_SYSC = SYSC_AUTOIDLE_MASK
            			| SYSC_ENAWAKEUP_MASK
            			| (SYSC_IDLEMODE_SMART << SYSC_IDLEMODE_SHIFT)
            			| (SYSC_CLOCKACTIVITY_FCLK << SYSC_CLOCKACTIVITY_SHIFT);
	
	/* Disable the controller */
	stI2cRegs->I2C_CON = 0;
	
	/* compute the PSC value */
	if (I2C_BUS_CLOCK > 100000)
	{
		/* I2C_FUNC_CLOCL = 48 Mhz
		 * I2C Internal Clock = 12 Mhz
		 * SCL values = (Internal Clock)/(2*(BUS Speed)) */
		stI2cRegs->I2C_PSC = 3;
		u16SCLValues = 12000000/I2C_BUS_CLOCK;
	}	
	else
	{
		/* I2C Functional clock = 48 Mhz
		 * I2C Internal clock = 4 Mhz
		 * SCL Values = (Internal Clock)/(2*(BUS Speed)) */
		stI2cRegs->I2C_PSC = 11;
		u16SCLValues = 4000000/I2C_BUS_CLOCK;
	}
	
	/* Disable  & clear all the Interrupts */
	stI2cRegs->I2C_IRQENABLE_CLR = 0x6FF;
	
	/* set the SCL Low and High time */
	stI2cRegs->I2C_SCLL = u16SCLValues - 7;
	stI2cRegs->I2C_SCLH = u16SCLValues - 5;
	
	/* Set the self address */
	stI2cRegs->I2C_OA = u16SlaveAddr & I2C_OA_MASK;
	
	/* Enable the I2C controller */
	stI2cRegs->I2C_CON = I2C_CON_EN;
	
	/* We don't operate in interrupt mode. So disable the interrupts */
	stI2cRegs->I2C_IE = 0;
	
	/* set the slave address */
	stI2cRegs->I2C_SA = 0x0;
	
	/* Set data count */
	stI2cRegs->I2C_CNT = 0x0;
	
	return (u32RetVal);

}
/** **************************************************************************
 * \n \brief Routine to read data from slave deice the I2C - 0
 *
 * This routine initializes the I2C controller for reading from the slave
 * peripheral connected to the bus. This function takes the slave address
 * as a parameter along with the read buffer pointer and the length of buffer.
 * This function returns the number of bytes read into the output len pointer.
 *
 * \param u8BusNum 		[IN]		I2C Bus number
 * \param u16SlaveAddr 	[IN]		I2C Slave address to read from
 * \param *u8Buffer		[OUT]		Buffer pointer to copy data into
 * \param u8BufLen 		[IN]		Length of the input buffer pointer
 * \param *u8DataRead	[OUT]		Place holder to return read number of bytes
 *
 * \return
 * \n      return SUCCESS for success  - Description
 * \n      return FAILED for error   - Description
 */

 INT32	DM388I2cRead
 (
	UINT8	u8BusNum,
	UINT16	u16SlaveAddr,
	UINT8	*u8Buffer,
	UINT8	u8BufLen,
	UINT8	*u8DataRead
 )
 {
	INT32 u32RetVal = SUCCESS;
	DM388_I2C_REGS	stI2cRegs = NULL;
	UINT16  u16Status = 0;
	UINT8	u8BytesRead = 0;
	UINT8*  pu8DataPtr = NULL;
	UINT32	u32TimeOut = DM388_I2C_TIMEOUT;
	
	/* input param validation */
	if ((NULL != u8Buffer) &&
		(0 != u8BufLen))
	{
		pu8DataPtr = u8Buffer;
	}
	else
	{
		u32RetVal = FAILED;
		return(u32RetVal);
	}

	switch (u8BusNum)
	{
		case 0:
			stI2cRegs = (DM388_I2C_REGS) I2C0_BASE;
			break;
		case 1:
			stI2cRegs = (DM388_I2C_REGS) I2C1_BASE;
			break;
		case 2:
			stI2cRegs = (DM388_I2C_REGS) I2C2_BASE;
			break;
		case 3:
			stI2cRegs = (DM388_I2C_REGS) I2C3_BASE;
			break;
		default:
			stI2cRegs = (DM388_I2C_REGS) NULL;
			platform_write ("DM388I2cRead: -Invalid I2C bus number \r\n");
			return FAILED;
	}
	
	/*  set the slave address */
	stI2cRegs->I2C_SA = u16SlaveAddr & I2C_SA_MASK;
	
	/* set the count */
	stI2cRegs->I2C_CNT = u8BufLen;
	
	/* Poll for the bus to become free/idle */
	for (u32TimeOut = 0; u32TimeOut < DM388_I2C_TIMEOUT; u32TimeOut++)
	{
		if ((stI2cRegs->I2C_STAT & I2C_STAT_BB) == 0)
			break;
	}
	
	if (u32TimeOut == DM388_I2C_TIMEOUT)
		u32RetVal = FAILED;	
	
	if (SUCCESS == u32RetVal)
	{
		/* configure the I2C bus as master for receiving mode */
		stI2cRegs->I2C_CON = 0	\
							| I2C_CON_EN    // I2C Enable
							| I2C_CON_MST       // Master Mode
							| I2C_CON_RX        // Receive Mode
							| I2C_CON_STP       // Stop
							| I2C_CON_STT       // Start
							;							
	}
	
	/* in a loop till we have read all the data or we have timed-out,
     * read the data from I2C data registetr and return
	 */
	for (u32TimeOut = 0; u32TimeOut < DM388_I2C_TIMEOUT; u32TimeOut++)
	{
		u16Status = stI2cRegs->I2C_STAT;
		
		/* Check for NACK*/
		if ((u16Status & I2C_STAT_NACK) == 0)
		{
			/*  ack received. Now check for data ready */
			if ((u16Status & I2C_STAT_ARDY) == 0)
			{
				/* check for RRDY for data availability */
				if ((u16Status & I2C_STAT_RRDY) == I2C_STAT_RRDY)
				{
					/* check if we have buffer space avaliable */
					if (u8BytesRead < u8BufLen)
					{
						/* data is available. Read it now and ack */
						*pu8DataPtr++ = stI2cRegs->I2C_DATA & 0xFF;
						u8BytesRead++;
					}
					/* clear the RRDY bits */
					stI2cRegs->I2C_STAT = I2C_STAT_RRDY;
				}
				else
				{
					/* loop through to make another attempt */
				}
			}
				
				/* if we have lost the arbitration */
			if ((stI2cRegs->I2C_STAT & I2C_STAT_AL) == I2C_STAT_AL)
				{
					platform_write ("DM388I2cRead: -AL... \r\n");
					u32RetVal = FAILED;
					/* clear the AL bits */
					stI2cRegs->I2C_STAT = I2C_STAT_AL;					
					break;
				}
			
			if ((stI2cRegs->I2C_STAT & I2C_STAT_ARDY) == I2C_STAT_ARDY)
				{
				/* all the reads are completed */
				stI2cRegs->I2C_STAT = I2C_STAT_ARDY;					
				break;
				}

			if ((stI2cRegs->I2C_STAT & I2C_STAT_NACK) == I2C_STAT_NACK)
				{
				platform_write ("DM388I2cRead: -NACK... \r\n");
				u32RetVal = FAILED;
				/* all the reads are completed */
				stI2cRegs->I2C_STAT = I2C_STAT_NACK;					
				break;
				}
				DM388_usecDelay( 1 );
				
		}
		else
		{
		platform_write ("DM388I2cRead: -NACK... \r\n");
		u32RetVal = FAILED;
		/* clear the NACK bits */
		stI2cRegs->I2C_STAT = I2C_STAT_NACK;					
		break;
		}
	}
	
	/* before returning, update the return counts too */
	if (NULL != u8DataRead)
		*u8DataRead = u8BytesRead;
		
	return (u32RetVal);
 }
/** **************************************************************************
 * \n \brief Routine to write data from slave deice the I2C - 0
 *
 * This routine initializes the I2C controller for writing to the slave
 * peripheral connected to the bus. This function takes the slave address
 * as a parameter along with the write buffer pointer and the length of buffer.
 * This function returns the number of bytes writen into the output len pointer.
 *
 * \param u8BusNum 		[IN]		I2C Bus number
 * \param u16SlaveAddr 	[IN]		I2C Slave address to read from
 * \param *u8Buffer		[OUT]		Buffer pointer to copy data into
 * \param u8BufLen 		[IN]		Length of the input buffer pointer
 * \param *u8DataWritten[OUT]		Place holder to return read number of bytes
 *
 * \return
 * \n      return SUCCESS for success  - Description
 * \n      return FAILED for error   - Description
 */
INT32 DM388I2cWite
(
	UINT8 	u8BusNum,
	UINT16 	u16SlaveAddr,
	UINT8	*u8Buffer,
	UINT8	u8BufLen,
	UINT8   *u8DataWritten
)
{
	INT32 u32RetVal = SUCCESS;
	DM388_I2C_REGS	stI2cRegs = NULL;
	UINT16  u16Status = 0;
	UINT8	u8BytesWritten = 0;
	UINT8*  pu8DataPtr = u8Buffer;
	UINT32	u32TimeOut = DM388_I2C_TIMEOUT;
	
	if ((NULL == pu8DataPtr) ||
		(0 == u8BufLen))
		{
		u32RetVal = 0;
		return (u32RetVal);
		}
		
	/* Assign the I2C register base */
	switch (u8BusNum)
	{
		case 0:
			stI2cRegs = (DM388_I2C_REGS) I2C0_BASE;
			break;
		case 1:
			stI2cRegs = (DM388_I2C_REGS) I2C1_BASE;
			break;
		case 2:
			stI2cRegs = (DM388_I2C_REGS) I2C2_BASE;
			break;
		case 3:
			stI2cRegs = (DM388_I2C_REGS) I2C3_BASE;
			break;
		default:
			stI2cRegs = (DM388_I2C_REGS) NULL;
			platform_write ("Invalid I2C bus number \r\n");
			return FAILED;
	}
	
	/* just to ensure the previous transactions flags are cleared */
	stI2cRegs->I2C_STAT = 0xFFFF;

	pu8DataPtr = u8Buffer;
	
	/*  set the slave address */
	stI2cRegs->I2C_SA = u16SlaveAddr & I2C_SA_MASK;
	
	/* set the count */
	stI2cRegs->I2C_CNT = u8BufLen;

	/* Poll for the bus to become free/idle */
	for (u32TimeOut = 0; u32TimeOut < DM388_I2C_TIMEOUT; u32TimeOut++)
	{
		if ((stI2cRegs->I2C_STAT & I2C_STAT_BB) == 0)
			break;
	}
	
	if (u32TimeOut == DM388_I2C_TIMEOUT)
		u32RetVal = FAILED;
		
	if (SUCCESS == u32RetVal)
	{
		/* configure the I2C bus as master for receiving mode */
		stI2cRegs->I2C_CON = 0	\
							| I2C_CON_EN    // I2C Enable
							| I2C_CON_MST       // Master Mode
							| I2C_CON_TX        // TRansmit Mode
							| I2C_CON_STP       // Stop
							| I2C_CON_STT       // Start
							;							
	}
	
	/* Loop till we have written all the data or we have timed-out */
	for (u32TimeOut = 0; u32TimeOut < DM388_I2C_TIMEOUT; u32TimeOut++)
	{
		u16Status = stI2cRegs->I2C_STAT;
		
		/* Check for NACK*/
		if ((u16Status & I2C_STAT_NACK) == 0)
		{
			/*  ack received. Now check for data ready */
			if ((u16Status & I2C_STAT_ARDY) == 0)
			{
				/* check for XRDY for data availability */
				if ((u16Status & I2C_STAT_XRDY) == I2C_STAT_XRDY)
				{
					/* clear the RRDY bits */
					stI2cRegs->I2C_STAT = I2C_STAT_XRDY;
					
					/* check if we have buffer space avaliable */
					if (u8BytesWritten < u8BufLen)
					{
						/* Write the data and ack */
						stI2cRegs->I2C_DATA = (*pu8DataPtr++ & 0xFF);
						u8BytesWritten++;
					}
				}
				else
				{
					/* loop through to make another attempt */
				}
			}
			
			if (u8BytesWritten == u8BufLen)
				DM388_usecDelay( 10 );
				
				
				/* if we have lost the arbitration */
			if ((stI2cRegs->I2C_STAT & I2C_STAT_AL) == I2C_STAT_AL)
				{
					platform_write ("DM388I2cWite:- AL... \r\n");
					u32RetVal = FAILED;
					/* clear the AL bits */
					stI2cRegs->I2C_STAT = I2C_STAT_AL;					
					break;
				}
			
			if ( (stI2cRegs->I2C_STAT & I2C_STAT_ARDY) == I2C_STAT_ARDY)
				{
				/* all the reads are completed */
				stI2cRegs->I2C_STAT = I2C_STAT_ARDY;					
				break;
				}

			if ( (stI2cRegs->I2C_STAT & I2C_STAT_NACK) == I2C_STAT_NACK)
				{
				platform_write ("DM388I2cWite:- NACK... \r\n");
				/* all the reads are completed */
				stI2cRegs->I2C_STAT = I2C_STAT_NACK;					
				break;
				}
				
			DM388_usecDelay( 10 );

		}
		else
		{
		platform_write ("DM388I2cWite:- NACK..... \r\n");
		u32RetVal = FAILED;
		/* clear the NACK bits */
		stI2cRegs->I2C_STAT = I2C_STAT_NACK;					
		break;
		}
	}
	
	/* before returning, update the return counts too */
	if (NULL != u8DataWritten)
		*u8DataWritten = u8BytesWritten;

	return (u32RetVal);
}

INT32 DM388I2cWitecamera
(
	UINT8 	u8BusNum,
	UINT16 	u16SlaveAddr,
	UINT8	*u8Buffer,
	UINT8	u8BufLen,
	UINT8   *u8DataWritten
)
{
	INT32 u32RetVal = SUCCESS;
	DM388_I2C_REGS	stI2cRegs = NULL;
	UINT16  u16Status = 0;
	UINT8	u8BytesWritten = 0;
	UINT8*  pu8DataPtr = u8Buffer;
	UINT32	u32TimeOut = DM388_I2C_TIMEOUT;
	UINT8 count=1;
	
	if ((NULL == pu8DataPtr) ||
		(0 == u8BufLen))
		{
		u32RetVal = 0;
		return (u32RetVal);
		}
		
	/* Assign the I2C register base */
	switch (u8BusNum)
	{
		case 0:
			stI2cRegs = (DM388_I2C_REGS) I2C0_BASE;
			break;
		case 1:
			stI2cRegs = (DM388_I2C_REGS) I2C1_BASE;
			break;
		case 2:
			stI2cRegs = (DM388_I2C_REGS) I2C2_BASE;
			break;
		case 3:
			stI2cRegs = (DM388_I2C_REGS) I2C3_BASE;
			break;
		default:
			stI2cRegs = (DM388_I2C_REGS) NULL;
			platform_write ("Invalid I2C bus number \r\n");
			return FAILED;
	}
	
	/* just to ensure the previous transactions flags are cleared */
	stI2cRegs->I2C_STAT = 0xFFFF;

	pu8DataPtr = u8Buffer;
	//platform_write("buffer data=%x %x\n",pu8DataPtr[0],pu8DataPtr[1]);
	//platform_write("u8Buffer address 0x%x \n",u8Buffer);

	
	/*  set the slave address */
	stI2cRegs->I2C_SA = u16SlaveAddr & I2C_SA_MASK;
	
	/* set the count */
	stI2cRegs->I2C_CNT = u8BufLen;

	/* Poll for the bus to become free/idle */
	for (u32TimeOut = 0; u32TimeOut < DM388_I2C_TIMEOUT; u32TimeOut++)
	{
		if ((stI2cRegs->I2C_STAT & I2C_STAT_BB) == 0)
			break;
	}
#if 0
	if (u32TimeOut == DM388_I2C_TIMEOUT)
	{
		u32RetVal = FAILED;
	}
#endif
	if (SUCCESS == u32RetVal)
	{
		/* configure the I2C bus as master for receiving mode */
		stI2cRegs->I2C_CON = 0	\
							| I2C_CON_EN    // I2C Enable
							| I2C_CON_MST       // Master Mode
							| I2C_CON_TX        // TRansmit Mode
							| I2C_CON_STP       // Stop
							| I2C_CON_STT       // Start
							;							
	}
	
	/* Loop till we have written all the data or we have timed-out */
	for (u32TimeOut = 0; u32TimeOut < DM388_I2C_TIMEOUT; u32TimeOut++)
	{
		u16Status = stI2cRegs->I2C_STAT;

		/* Check for NACK*/
		if ((u16Status & I2C_STAT_NACK) == 0)
		{
			/*  ack received. Now check for data ready */
			if ((u16Status & I2C_STAT_ARDY) == 0)
			{
				/* check for XRDY for data availability */
				if ((u16Status & I2C_STAT_XRDY) == I2C_STAT_XRDY)
				{
					/* clear the RRDY bits */
					stI2cRegs->I2C_STAT = I2C_STAT_XRDY;
					
					/* check if we have buffer space avaliable */

					if (u8BytesWritten < u8BufLen)
					{
						/* Write the data and ack */
						//platform_write("pu8DataPtr address- 0x%x and pu8DataPtr data=0x%x\n",pu8DataPtr,*pu8DataPtr);
						stI2cRegs->I2C_DATA = (*pu8DataPtr++ & 0xFF);
						count++;
						u8BytesWritten++;

					}
				}
				else
				{
					/* loop through to make another attempt */
				}
			}
			
			if (u8BytesWritten == u8BufLen)
			{
				//DM388_usecDelay( 10 );
				break;
			}
				
				/* if we have lost the arbitration */
			if ((stI2cRegs->I2C_STAT & I2C_STAT_AL) == I2C_STAT_AL)
				{
					platform_write ("DM388I2cWite:- AL... \r\n");
					u32RetVal = FAILED;
					/* clear the AL bits */
					stI2cRegs->I2C_STAT = I2C_STAT_AL;					
					break;
				}
			#if 1
			if ( (stI2cRegs->I2C_STAT & I2C_STAT_ARDY) == I2C_STAT_ARDY)
				{
				/* all the reads are completed */
				stI2cRegs->I2C_STAT = I2C_STAT_ARDY;					
				break;
				}
			#endif
			if ( (stI2cRegs->I2C_STAT & I2C_STAT_NACK) == I2C_STAT_NACK)
				{
				platform_write ("DM388I2cWite:- NACK... \r\n");
				/* all the reads are completed */
				stI2cRegs->I2C_STAT = I2C_STAT_NACK;					
				break;
				}
				
			 DM388_usecDelay( 10 );

		}

		else
		{
		platform_write ("DM388I2cWite:- NACK..... \r\n");
		u32RetVal = FAILED;
		/* clear the NACK bits */
		stI2cRegs->I2C_STAT = I2C_STAT_NACK;					
		break;
		}
	}
	
	/* before returning, update the return counts too */
	if (NULL != u8DataWritten)
		*u8DataWritten = u8BytesWritten;

	return (u32RetVal);
}

/** **************************************************************************
 * \n \brief Routine to Probe slave device on the I2C Bus
 *
 * This routine tries to write a byte to the slave device and looks for NACk.
 * If there is no NACK, it assumes the device is present.
 *
 * \param u8BusNum 		[IN]		I2C Bus number
 * \param u16SlaveAddr 	[IN]		I2C Slave address to read from
 *
 * \return
 * \n      return SUCCESS for success  - Description
 * \n      return FAILED for error   - Description
 */
INT32 DM388I2cProbe
(
	UINT8 	u8BusNum,
	UINT16 	u16SlaveAddr
)
{
	INT32 u32RetVal = SUCCESS;
	DM388_I2C_REGS	stI2cRegs = NULL;
	UINT16  u16Status = 0;
	UINT8	u8BytesWritten = 0;
	UINT32	u32TimeOut = DM388_I2C_TIMEOUT;

	switch (u8BusNum)
	{
		case 0:
			stI2cRegs = (DM388_I2C_REGS) I2C0_BASE;
			break;
		case 1:
			stI2cRegs = (DM388_I2C_REGS) I2C1_BASE;
			break;
		case 2:
			stI2cRegs = (DM388_I2C_REGS) I2C2_BASE;
			break;
		case 3:
			stI2cRegs = (DM388_I2C_REGS) I2C3_BASE;
			break;
		default:
			stI2cRegs = (DM388_I2C_REGS) NULL;
			platform_write ("Invalid I2C bus number \r\n");
			return FAILED;
	}

	/* just to ensure the previous transactions flags are cleared */
	stI2cRegs->I2C_STAT = 0xFFFF;
	
	/* Disable the Interrupts */
	stI2cRegs->I2C_IE = 0x00;

	/* Poll for the bus to become free/idle */
	for (u32TimeOut = 0; u32TimeOut < DM388_I2C_TIMEOUT; u32TimeOut++)
	{
		if ((stI2cRegs->I2C_STAT & I2C_STAT_BB) == 0)
			break;
	}
	
	if (u32TimeOut == DM388_I2C_TIMEOUT)
	{
		platform_write ("Bus is busy.\r\n");
		u32RetVal = FAILED;
		return (u32RetVal);
	}


	/* set the count */
	stI2cRegs->I2C_CNT = 1;

	/*  set the slave address */
	stI2cRegs->I2C_SA = u16SlaveAddr & I2C_SA_MASK;
		
	if (SUCCESS == u32RetVal)
	{
		/* configure the I2C bus as master for receiving mode */
		stI2cRegs->I2C_CON = 0	\
							| I2C_CON_EN    // I2C Enable
							| I2C_CON_MST       // Master Mode
							| I2C_CON_STP       // Stop
							| I2C_CON_STT       // Start
							| I2C_CON_RX        // Receive Mode							
							;							
	}
	
	u32RetVal = FAILED;

	/* have a large enough delay for client device to respond */
	DM388_usecDelay (2 * DM388_I2C_TIMEOUT);
	
	/* check the status to see if we succeeded */
	u16Status = stI2cRegs->I2C_STAT;
	
	if (I2C_STAT_NACK == (u16Status & I2C_STAT_NACK))
	{
		u32RetVal = FAILED;
		/* clear the status bits */
		stI2cRegs->I2C_STAT = 0xFFFF;
		/* configure the I2C bus as master for receiving mode */
		stI2cRegs->I2C_CON |= I2C_CON_STP;       // Stop		

		DM388_usecDelay (1000);
		/* Poll for the bus to become free/idle */
		for (u32TimeOut = 0; u32TimeOut < DM388_I2C_TIMEOUT; u32TimeOut++)
		{
			if ((stI2cRegs->I2C_STAT & I2C_STAT_BB) == 0)
				break;
		}		
	}
	else
	{
		/* We got the ACK from the device. So clear the I2C status */
		for (u32TimeOut = 0; u32TimeOut < DM388_I2C_TIMEOUT; u32TimeOut++)
		{
			u16Status = stI2cRegs->I2C_STAT;
			
			if(I2C_STAT_RRDY == (u16Status & I2C_STAT_RRDY))
			{
				u32RetVal = SUCCESS;
				u8BytesWritten = stI2cRegs->I2C_DATA;
				stI2cRegs->I2C_STAT = I2C_STAT_RRDY;
				DM388_usecDelay (1000);
				break;
			}
			else
				break;
		}
	}
	
	/* clear the status bits */
	stI2cRegs->I2C_STAT = 0xFFFF;
	/* configure the I2C bus as master for receiving mode */
	stI2cRegs->I2C_CON |= I2C_CON_STP;       // Stop
	
	DM388_usecDelay (10);
	
	/* to avoide the compiler warning */
	u8BytesWritten = u8BytesWritten ;
	
	return (u32RetVal);
}


/** **************************************************************************
 * \n \brief Routine to write data to slave deice siI9135 through I2C - 2
 *
 * This routine initializes the I2C controller for writing to the slave
 * peripheral connected to the bus. This function takes the slave address
 * as a parameter along with the write buffer pointer and the length of buffer.
 * This function returns the number of bytes writen into the output len pointer.
 *
 * \param u8BusNum 		[IN]		I2C Bus number
 * \param u16SlaveAddr 	[IN]		I2C Slave address to read from
 * \param *u8Buffer		[OUT]		Buffer pointer to copy data into
 * \param u8BufLen 		[IN]		Length of the input buffer pointer
 * \param *u8DataWritten[OUT]		Place holder to return read number of bytes
 *
 * \return
 * \n      return SUCCESS for success  - Description
 * \n      return FAILED for error   - Description
 */
INT32 siI9135I2cWite
(
	UINT8 	u8BusNum,
	UINT16 	u16SlaveAddr,
	UINT8	*u8Buffer,
	UINT8	u8BufLen,
	UINT8   *u8DataWritten
)
{
	INT32 u32RetVal = SUCCESS;
	DM388_I2C_REGS	stI2cRegs = NULL;
	UINT16  u16Status = 0;
	UINT8	u8BytesWritten = 0;
	UINT8*  pu8DataPtr = u8Buffer;
	UINT32	u32TimeOut = DM388_I2C_TIMEOUT;
	
	if ((NULL == pu8DataPtr) ||
		(0 == u8BufLen))
		{
		u32RetVal = 0;
		return (u32RetVal);
		}
		
	/* Assign the I2C register base */
	switch (u8BusNum)
	{
		case 0:
			stI2cRegs = (DM388_I2C_REGS) I2C0_BASE;
			break;
		case 1:
			stI2cRegs = (DM388_I2C_REGS) I2C1_BASE;
			break;
		case 2:
			stI2cRegs = (DM388_I2C_REGS) I2C2_BASE;
			break;
		case 3:
			stI2cRegs = (DM388_I2C_REGS) I2C3_BASE;
			break;
		default:
			stI2cRegs = (DM388_I2C_REGS) NULL;
			platform_write ("Invalid I2C bus number \r\n");
			return FAILED;
	}
	
	/* just to ensure the previous transactions flags are cleared */
	stI2cRegs->I2C_STAT = 0xFFFF;

	pu8DataPtr = u8Buffer;
	
	/*  set the slave address */
	stI2cRegs->I2C_SA = u16SlaveAddr & I2C_SA_MASK;
	
	/* set the count */
	stI2cRegs->I2C_CNT = u8BufLen;

	/* Poll for the bus to become free/idle */
	for (u32TimeOut = 0; u32TimeOut < DM388_I2C_TIMEOUT; u32TimeOut++)
	{
		if ((stI2cRegs->I2C_STAT & I2C_STAT_BB) == 0)
			break;
	}
	
	if (u32TimeOut == DM388_I2C_TIMEOUT)
		u32RetVal = FAILED;
		
	if (SUCCESS == u32RetVal)
	{
		/* configure the I2C bus as master for receiving mode */
		stI2cRegs->I2C_CON = 0	\
							| I2C_CON_EN    // I2C Enable
							| I2C_CON_MST       // Master Mode
							| I2C_CON_TX        // TRansmit Mode
							| I2C_CON_STT       // Start
							;							
	}
	
	/* Loop till we have written all the data or we have timed-out */
	for (u32TimeOut = 0; u32TimeOut < DM388_I2C_TIMEOUT; u32TimeOut++)
	{
		u16Status = stI2cRegs->I2C_STAT;
		
		/* Check for NACK*/
		if ((u16Status & I2C_STAT_NACK) == 0)
		{
			/*  ack received. Now check for data ready */
			if ((u16Status & I2C_STAT_ARDY) == 0)
			{
				/* check for XRDY for data availability */
				if ((u16Status & I2C_STAT_XRDY) == I2C_STAT_XRDY)
				{
					/* clear the RRDY bits */
					stI2cRegs->I2C_STAT = I2C_STAT_XRDY;
					
					/* check if we have buffer space avaliable */
					if (u8BytesWritten < u8BufLen)
					{
						/* Write the data and ack */
						stI2cRegs->I2C_DATA = (*pu8DataPtr++ & 0xFF);
						u8BytesWritten++;
					}
				}
				else
				{
					/* loop through to make another attempt */
				}
			}
			
			if (u8BytesWritten == u8BufLen)
				DM388_usecDelay( 10 );
				
				
				/* if we have lost the arbitration */
			if ((stI2cRegs->I2C_STAT & I2C_STAT_AL) == I2C_STAT_AL)
				{
					platform_write ("DM388I2cWite:- AL... \r\n");
					u32RetVal = FAILED;
					/* clear the AL bits */
					stI2cRegs->I2C_STAT = I2C_STAT_AL;					
					break;
				}
			
			if ( (stI2cRegs->I2C_STAT & I2C_STAT_ARDY) == I2C_STAT_ARDY)
				{
				/* all the reads are completed */
				stI2cRegs->I2C_STAT = I2C_STAT_ARDY;					
				break;
				}

			if ( (stI2cRegs->I2C_STAT & I2C_STAT_NACK) == I2C_STAT_NACK)
				{
				platform_write ("DM388I2cWite:- NACK... \r\n");
				/* all the reads are completed */
				stI2cRegs->I2C_STAT = I2C_STAT_NACK;					
				break;
				}
				
			DM388_usecDelay( 10 );

		}
		else
		{
		platform_write ("DM388I2cWite:- NACK..... \r\n");
		u32RetVal = FAILED;
		/* clear the NACK bits */
		stI2cRegs->I2C_STAT = I2C_STAT_NACK;					
		break;
		}
	}
	
	/* before returning, update the return counts too */
	if (NULL != u8DataWritten)
		*u8DataWritten = u8BytesWritten;

	return (u32RetVal);
}

/** **************************************************************************
 * \n \brief Routine to read data from slave deice siI9135 using I2C - 2
 *
 * This routine initializes the I2C controller for reading from the slave
 * peripheral connected to the bus. This function takes the slave address
 * as a parameter along with the read buffer pointer and the length of buffer.
 * This function returns the number of bytes read into the output len pointer.
 *
 * \param u8BusNum 		[IN]		I2C Bus number
 * \param u16SlaveAddr 	[IN]		I2C Slave address to read from
 * \param *u8Buffer		[OUT]		Buffer pointer to copy data into
 * \param u8BufLen 		[IN]		Length of the input buffer pointer
 * \param *u8DataRead	[OUT]		Place holder to return read number of bytes
 *
 * \return
 * \n      return SUCCESS for success  - Description
 * \n      return FAILED for error   - Description
 */

 INT32	siI9135I2cRead
 (
	UINT8	u8BusNum,
	UINT16	u16SlaveAddr,
	UINT8	*u8Buffer,
	UINT8	u8BufLen,
	UINT8	*u8DataRead
 )
 {
	INT32 u32RetVal = SUCCESS;
	DM388_I2C_REGS	stI2cRegs = NULL;
	UINT16  u16Status = 0;
	UINT8	u8BytesRead = 0;
	UINT8*  pu8DataPtr = NULL;
	UINT32	u32TimeOut = DM388_I2C_TIMEOUT;
	
	/* input param validation */
	if ((NULL != u8Buffer) &&
		(0 != u8BufLen))
	{
		pu8DataPtr = u8Buffer;
	}
	else
	{
		u32RetVal = FAILED;
		return(u32RetVal);
	}

	switch (u8BusNum)
	{
		case 0:
			stI2cRegs = (DM388_I2C_REGS) I2C0_BASE;
			break;
		case 1:
			stI2cRegs = (DM388_I2C_REGS) I2C1_BASE;
			break;
		case 2:
			stI2cRegs = (DM388_I2C_REGS) I2C2_BASE;
			break;
		case 3:
			stI2cRegs = (DM388_I2C_REGS) I2C3_BASE;
			break;
		default:
			stI2cRegs = (DM388_I2C_REGS) NULL;
			platform_write ("DM388I2cRead: -Invalid I2C bus number \r\n");
			return FAILED;
	}
	
	/*  set the slave address */
	stI2cRegs->I2C_SA = u16SlaveAddr & I2C_SA_MASK;
	
	/* set the count */
	stI2cRegs->I2C_CNT = u8BufLen;
	
	/* Poll for the bus to become free/idle */
	for (u32TimeOut = 0; u32TimeOut < DM388_I2C_TIMEOUT; u32TimeOut++)
	{
		if ((stI2cRegs->I2C_STAT & I2C_STAT_BB) == 0)
			break;
	}

	if (SUCCESS == u32RetVal)
	{
		/* configure the I2C bus as master for receiving mode */
		stI2cRegs->I2C_CON = 0	\
							| I2C_CON_EN    // I2C Enable
							| I2C_CON_MST       // Master Mode
							| I2C_CON_RX        // Receive Mode
							| I2C_CON_STP       // Stop
							| I2C_CON_STT       // Start
							;							
	}
	
	/* in a loop till we have read all the data or we have timed-out,
     * read the data from I2C data registetr and return
	 */
	for (u32TimeOut = 0; u32TimeOut < DM388_I2C_TIMEOUT; u32TimeOut++)
	{
		u16Status = stI2cRegs->I2C_STAT;
		
		/* Check for NACK*/
		if ((u16Status & I2C_STAT_NACK) == 0)
		{
			/*  ack received. Now check for data ready */
			if ((u16Status & I2C_STAT_ARDY) == 0)
			{
				/* check for RRDY for data availability */
				if ((u16Status & I2C_STAT_RRDY) == I2C_STAT_RRDY)
				{
					/* check if we have buffer space avaliable */
					if (u8BytesRead < u8BufLen)
					{
						/* data is available. Read it now and ack */
						*pu8DataPtr++ = stI2cRegs->I2C_DATA & 0xFF;
						u8BytesRead++;
					}
					/* clear the RRDY bits */
					stI2cRegs->I2C_STAT = I2C_STAT_RRDY;
				}
				else
				{
					/* loop through to make another attempt */
				}
			}
				
				/* if we have lost the arbitration */
			if ((stI2cRegs->I2C_STAT & I2C_STAT_AL) == I2C_STAT_AL)
				{
					platform_write ("DM388I2cRead: -AL... \r\n");
					u32RetVal = FAILED;
					/* clear the AL bits */
					stI2cRegs->I2C_STAT = I2C_STAT_AL;					
					break;
				}
			
			if ((stI2cRegs->I2C_STAT & I2C_STAT_ARDY) == I2C_STAT_ARDY)
				{
				/* all the reads are completed */
				stI2cRegs->I2C_STAT = I2C_STAT_ARDY;					
				break;
				}

			if ((stI2cRegs->I2C_STAT & I2C_STAT_NACK) == I2C_STAT_NACK)
				{
				platform_write ("DM388I2cRead: -NACK... \r\n");
				u32RetVal = FAILED;
				/* all the reads are completed */
				stI2cRegs->I2C_STAT = I2C_STAT_NACK;					
				break;
				}
				DM388_usecDelay( 1 );
				
		}
		else
		{
		platform_write ("DM388I2cRead: -NACK... \r\n");
		u32RetVal = FAILED;
		/* clear the NACK bits */
		stI2cRegs->I2C_STAT = I2C_STAT_NACK;					
		break;
		}
	}
	
	/* before returning, update the return counts too */
	if (NULL != u8DataRead)
		*u8DataRead = u8BytesRead;
		
	return (u32RetVal);
 }
